use super::schema::{member_settlement_settings, ManualPaymentType, SettlementType};
use beteran_common_rust as bcr;

///
#[derive(Identifiable, Queryable, PartialEq, PartialOrd, Debug, Clone)]
#[table_name = "member_settlement_settings"]
pub struct MemberSettlementSetting {
  ///
  pub id: uuid::Uuid,
  ///
  pub member_id: uuid::Uuid,
  ///
  pub can_exchange: bool,
  ///
  pub can_first_charge_comp: bool,
  ///
  pub can_per_charge_comp: bool,
  ///
  pub manual_payment_type_for_partner: ManualPaymentType,
  ///
  pub settlement_type: SettlementType,
  ///
  pub rate_casino: f64,
  ///
  pub rate_casino_loosing: f64,
  ///
  pub rate_casino_bacara: f64,
  ///
  pub rate_casino_roulette: f64,
  ///
  pub rate_casino_dragon_tiger: f64,
  ///
  pub rate_slot: f64,
  ///
  pub rate_slot_loosing: f64,
  ///
  pub rate_powerball_single: f64,
  ///
  pub rate_powerball_combo: f64,
  ///
  pub rate_powerladder_single: f64,
  ///
  pub rate_powerladder_combo: f64,
  ///
  pub rate_eos_single: f64,
  ///
  pub rate_eos_combo: f64,
  ///
  pub rate_bogglepowerball_single: f64,
  ///
  pub rate_bogglepowerball_combo: f64,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}

///
#[derive(Insertable, Debug, Clone)]
#[table_name = "member_settlement_settings"]
pub struct NewMemberSettlementSetting {
  ///
  pub member_id: uuid::Uuid,
  ///
  pub can_exchange: bool,
  ///
  pub can_first_charge_comp: bool,
  ///
  pub can_per_charge_comp: bool,
  ///
  pub manual_payment_type_for_partner: ManualPaymentType,
  ///
  pub settlement_type: SettlementType,
  ///
  pub rate_casino: f64,
  ///
  pub rate_casino_loosing: f64,
  ///
  pub rate_casino_bacara: f64,
  ///
  pub rate_casino_roulette: f64,
  ///
  pub rate_casino_dragon_tiger: f64,
  ///
  pub rate_slot: f64,
  ///
  pub rate_slot_loosing: f64,
  ///
  pub rate_powerball_single: f64,
  ///
  pub rate_powerball_combo: f64,
  ///
  pub rate_powerladder_single: f64,
  ///
  pub rate_powerladder_combo: f64,
  ///
  pub rate_eos_single: f64,
  ///
  pub rate_eos_combo: f64,
  ///
  pub rate_bogglepowerball_single: f64,
  ///
  pub rate_bogglepowerball_combo: f64,
}

///
#[derive(AsChangeset, Debug, Clone)]
#[table_name = "member_settlement_settings"]
pub struct ModifyMemberSettlementSetting {
  ///
  pub can_exchange: bool,
  ///
  pub can_first_charge_comp: bool,
  ///
  pub can_per_charge_comp: bool,
  ///
  pub manual_payment_type_for_partner: ManualPaymentType,
  ///
  pub settlement_type: SettlementType,
  ///
  pub rate_casino: f64,
  ///
  pub rate_casino_loosing: f64,
  ///
  pub rate_casino_bacara: f64,
  ///
  pub rate_casino_roulette: f64,
  ///
  pub rate_casino_dragon_tiger: f64,
  ///
  pub rate_slot: f64,
  ///
  pub rate_slot_loosing: f64,
  ///
  pub rate_powerball_single: f64,
  ///
  pub rate_powerball_combo: f64,
  ///
  pub rate_powerladder_single: f64,
  ///
  pub rate_powerladder_combo: f64,
  ///
  pub rate_eos_single: f64,
  ///
  pub rate_eos_combo: f64,
  ///
  pub rate_bogglepowerball_single: f64,
  ///
  pub rate_bogglepowerball_combo: f64,
}

///
#[derive(Debug, Clone)]
pub struct FindAllSearch {
  ///
  pub member_id: Option<uuid::Uuid>,
}

///
#[derive(Debug, Clone)]
pub struct FindAll {
  ///
  pub search: Option<FindAllSearch>,
  ///
  pub pagination: Option<bcr::models::pagination::Pagination>,
  ///
  pub sorts: Option<Vec<bcr::models::pagination::Sort>>,
}
