//!
//!
use super::{models, schema::member_settlement_settings};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of member_settlement_settings")
      .finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_resource: &models::NewMemberSettlementSetting,
  ) -> Result<models::MemberSettlementSetting, Error> {
    let inserted = diesel::insert_into(member_settlement_settings::table)
      .values(new_resource)
      .get_result::<models::MemberSettlementSetting>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::MemberSettlementSetting>, Error> {
    match member_settlement_settings::table
      .find(id as uuid::Uuid)
      .first::<models::MemberSettlementSetting>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::MemberSettlementSetting>, Error> {
    use member_settlement_settings::dsl;

    match member_settlement_settings::table
      .filter(dsl::member_id.eq(member_id))
      .first::<models::MemberSettlementSetting>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = member_settlement_settings::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_settlement_settings::dsl::member_id.eq(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::MemberSettlementSetting>, Error> {
    let mut q = member_settlement_settings::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_settlement_settings::dsl::member_id.eq(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "rate_casino" => {
              q = q.order_by(member_settlement_settings::rate_casino.asc());
            }
            "rate_casino_loosing" => {
              q = q.order_by(member_settlement_settings::rate_casino_loosing.asc());
            }
            "rate_casino_bacara" => {
              q = q.order_by(member_settlement_settings::rate_casino_bacara.asc());
            }
            "rate_casino_roulette" => {
              q = q.order_by(member_settlement_settings::rate_casino_roulette.asc());
            }
            "rate_casino_dragon_tiger" => {
              q = q.order_by(member_settlement_settings::rate_casino_dragon_tiger.asc());
            }
            "rate_slot" => {
              q = q.order_by(member_settlement_settings::rate_slot.asc());
            }
            "rate_slot_loosing" => {
              q = q.order_by(member_settlement_settings::rate_slot_loosing.asc());
            }
            "rate_powerball_single" => {
              q = q.order_by(member_settlement_settings::rate_powerball_single.asc());
            }
            "rate_powerball_combo" => {
              q = q.order_by(member_settlement_settings::rate_powerball_combo.asc());
            }
            "rate_powerladder_single" => {
              q = q.order_by(member_settlement_settings::rate_powerladder_single.asc());
            }
            "rate_powerladder_combo" => {
              q = q.order_by(member_settlement_settings::rate_powerladder_combo.asc());
            }
            "rate_eos_single" => {
              q = q.order_by(member_settlement_settings::rate_eos_single.asc());
            }
            "rate_eos_combo" => {
              q = q.order_by(member_settlement_settings::rate_eos_combo.asc());
            }
            "rate_bogglepowerball_single" => {
              q = q.order_by(member_settlement_settings::rate_bogglepowerball_single.asc());
            }
            "rate_bogglepowerball_combo" => {
              q = q.order_by(member_settlement_settings::rate_bogglepowerball_combo.asc());
            }
            "created_at" => {
              q = q.order_by(member_settlement_settings::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(member_settlement_settings::updated_at.asc());
            }
            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "rate_casino" => {
              q = q.order_by(member_settlement_settings::rate_casino.desc());
            }
            "rate_casino_loosing" => {
              q = q.order_by(member_settlement_settings::rate_casino_loosing.desc());
            }
            "rate_casino_bacara" => {
              q = q.order_by(member_settlement_settings::rate_casino_bacara.desc());
            }
            "rate_casino_roulette" => {
              q = q.order_by(member_settlement_settings::rate_casino_roulette.desc());
            }
            "rate_casino_dragon_tiger" => {
              q = q.order_by(member_settlement_settings::rate_casino_dragon_tiger.desc());
            }
            "rate_slot" => {
              q = q.order_by(member_settlement_settings::rate_slot.desc());
            }
            "rate_slot_loosing" => {
              q = q.order_by(member_settlement_settings::rate_slot_loosing.desc());
            }
            "rate_powerball_single" => {
              q = q.order_by(member_settlement_settings::rate_powerball_single.desc());
            }
            "rate_powerball_combo" => {
              q = q.order_by(member_settlement_settings::rate_powerball_combo.desc());
            }
            "rate_powerladder_single" => {
              q = q.order_by(member_settlement_settings::rate_powerladder_single.desc());
            }
            "rate_powerladder_combo" => {
              q = q.order_by(member_settlement_settings::rate_powerladder_combo.desc());
            }
            "rate_eos_single" => {
              q = q.order_by(member_settlement_settings::rate_eos_single.desc());
            }
            "rate_eos_combo" => {
              q = q.order_by(member_settlement_settings::rate_eos_combo.desc());
            }
            "rate_bogglepowerball_single" => {
              q = q.order_by(member_settlement_settings::rate_bogglepowerball_single.desc());
            }
            "rate_bogglepowerball_combo" => {
              q = q.order_by(member_settlement_settings::rate_bogglepowerball_combo.desc());
            }
            "created_at" => {
              q = q.order_by(member_settlement_settings::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(member_settlement_settings::updated_at.desc());
            }

            _ => {}
          },
        };
      }
    }

    q.load::<models::MemberSettlementSetting>(conn)
  }

  ///
  pub fn update(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifyMemberSettlementSetting,
  ) -> Result<u64, Error> {
    use member_settlement_settings::dsl;

    diesel::update(dsl::member_settlement_settings.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: uuid::Uuid) -> Result<u64, Error> {
    use member_settlement_settings::dsl;

    diesel::delete(member_settlement_settings::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
